@@ -16,8 +16,10 @@ python3 -m uv run python -c 'import docs.plugins.main'
 #   pydantic_core/core_schema.py
 # instead of
 #   .venv/lib/python3.10/site-packages/pydantic_core/core_schema.py
+# See also: mkdocs.yml:mkdocstrings:handlers:python:paths: [.]:
 ln -s .venv/lib/python*/site-packages/pydantic_core pydantic_core
 ln -s .venv/lib/python*/site-packages/pydantic_settings pydantic_settings
 ln -s .venv/lib/python*/site-packages/pydantic_extra_types pydantic_extra_types
-
-python3 -m uv run --no-sync mkdocs build
+# Put these at the front of PYTHONPATH (otherwise, symlinked
+# entries will still have "Source code in .venv/lib/.../*.py ":
+PYTHONPATH="$PWD${PYTHONPATH:+:${PYTHONPATH}}" python3 -m uv run --no-sync mkdocs build
