@@ -2,14 +2,16 @@
 
 from __future__ import annotations as _annotations
 
+import warnings
 from re import Pattern
-from typing import TYPE_CHECKING, Any, Callable, Dict, List, Type, TypeVar, Union
+from typing import TYPE_CHECKING, Any, Callable, Literal, TypeVar, Union, cast, overload
 
-from typing_extensions import Literal, TypeAlias, TypedDict
+from typing_extensions import TypeAlias, TypedDict, Unpack, deprecated
 
 from ._migration import getattr_migration
 from .aliases import AliasGenerator
 from .errors import PydanticUserError
+from .warnings import PydanticDeprecatedSince211
 
 if TYPE_CHECKING:
     from ._internal._generate_schema import GenerateSchema as _GenerateSchema
@@ -18,14 +20,14 @@ if TYPE_CHECKING:
 __all__ = ('ConfigDict', 'with_config')
 
 
-JsonValue: TypeAlias = Union[int, float, str, bool, None, List['JsonValue'], 'JsonDict']
-JsonDict: TypeAlias = Dict[str, JsonValue]
+JsonValue: TypeAlias = Union[int, float, str, bool, None, list['JsonValue'], 'JsonDict']
+JsonDict: TypeAlias = dict[str, JsonValue]
 
 JsonEncoder = Callable[[Any], Any]
 
 JsonSchemaExtraCallable: TypeAlias = Union[
     Callable[[JsonDict], None],
-    Callable[[JsonDict, Type[Any]], None],
+    Callable[[JsonDict, type[Any]], None],
 ]
 
 ExtraValues = Literal['allow', 'ignore', 'forbid']
@@ -59,69 +61,120 @@ class ConfigDict(TypedDict, total=False):
     """The maximum length for str types. Defaults to `None`."""
 
     extra: ExtraValues | None
-    """
-    Whether to ignore, allow, or forbid extra attributes during model initialization. Defaults to `'ignore'`.
+    '''
+    Whether to ignore, allow, or forbid extra data during model initialization. Defaults to `'ignore'`.
 
-    You can configure how pydantic handles the attributes that are not defined in the model:
+    Three configuration values are available:
 
-    * `allow` - Allow any extra attributes.
-    * `forbid` - Forbid any extra attributes.
-    * `ignore` - Ignore any extra attributes.
+    - `'ignore'`: Providing extra data is ignored (the default):
+      ```python
+      from pydantic import BaseModel, ConfigDict
 
-    ```python
-    from pydantic import BaseModel, ConfigDict
+      class User(BaseModel):
+          model_config = ConfigDict(extra='ignore')  # (1)!
 
-    class User(BaseModel):
-        model_config = ConfigDict(extra='ignore')  # (1)!
+          name: str
 
-        name: str
+      user = User(name='John Doe', age=20)  # (2)!
+      print(user)
+      #> name='John Doe'
+      ```
 
-    user = User(name='John Doe', age=20)  # (2)!
-    print(user)
-    #> name='John Doe'
-    ```
+        1. This is the default behaviour.
+        2. The `age` argument is ignored.
 
-    1. This is the default behaviour.
-    2. The `age` argument is ignored.
+    - `'forbid'`: Providing extra data is not permitted, and a [`ValidationError`][pydantic_core.ValidationError]
+      will be raised if this is the case:
+      ```python
+      from pydantic import BaseModel, ConfigDict, ValidationError
 
-    Instead, with `extra='allow'`, the `age` argument is included:
 
-    ```python
-    from pydantic import BaseModel, ConfigDict
+      class Model(BaseModel):
+          x: int
 
-    class User(BaseModel):
-        model_config = ConfigDict(extra='allow')
+          model_config = ConfigDict(extra='forbid')
 
-        name: str
 
-    user = User(name='John Doe', age=20)  # (1)!
-    print(user)
-    #> name='John Doe' age=20
-    ```
+      try:
+          Model(x=1, y='a')
+      except ValidationError as exc:
+          print(exc)
+          """
+          1 validation error for Model
+          y
+            Extra inputs are not permitted [type=extra_forbidden, input_value='a', input_type=str]
+          """
+      ```
 
-    1. The `age` argument is included.
+    - `'allow'`: Providing extra data is allowed and stored in the `__pydantic_extra__` dictionary attribute:
+      ```python
+      from pydantic import BaseModel, ConfigDict
 
-    With `extra='forbid'`, an error is raised:
 
+      class Model(BaseModel):
+          x: int
+
+          model_config = ConfigDict(extra='allow')
+
+
+      m = Model(x=1, y='a')
+      assert m.__pydantic_extra__ == {'y': 'a'}
+      ```
+      By default, no validation will be applied to these extra items, but you can set a type for the values by overriding
+      the type annotation for `__pydantic_extra__`:
+      ```python
+      from pydantic import BaseModel, ConfigDict, Field, ValidationError
+
+
+      class Model(BaseModel):
+          __pydantic_extra__: dict[str, int] = Field(init=False)  # (1)!
+
+          x: int
+
+          model_config = ConfigDict(extra='allow')
+
+
+      try:
+          Model(x=1, y='a')
+      except ValidationError as exc:
+          print(exc)
+          """
+          1 validation error for Model
+          y
+            Input should be a valid integer, unable to parse string as an integer [type=int_parsing, input_value='a', input_type=str]
+          """
+
+      m = Model(x=1, y='2')
+      assert m.x == 1
+      assert m.y == 2
+      assert m.model_dump() == {'x': 1, 'y': 2}
+      assert m.__pydantic_extra__ == {'y': 2}
+      ```
+
+        1. The `= Field(init=False)` does not have any effect at runtime, but prevents the `__pydantic_extra__` field from
+           being included as a parameter to the model's `__init__` method by type checkers.
+
+    As well as specifying an `extra` configuration value on the model, you can also provide it as an argument to the validation methods.
+    This will override any `extra` configuration value set on the model:
     ```python
     from pydantic import BaseModel, ConfigDict, ValidationError
 
-    class User(BaseModel):
-        model_config = ConfigDict(extra='forbid')
-
-        name: str
+    class Model(BaseModel):
+        x: int
+        model_config = ConfigDict(extra="allow")
 
     try:
-        User(name='John Doe', age=20)
-    except ValidationError as e:
-        print(e)
-        '''
-        1 validation error for User
-        age
-          Extra inputs are not permitted [type=extra_forbidden, input_value=20, input_type=int]
-        '''
+        # Override model config and forbid extra fields just this time
+        Model.model_validate({"x": 1, "y": 2}, extra="forbid")
+    except ValidationError as exc:
+        print(exc)
+        """
+        1 validation error for Model
+        y
+          Extra inputs are not permitted [type=extra_forbidden, input_value=2, input_type=int]
+        """
     ```
-    """
+    '''
 
     frozen: bool
     """
@@ -138,30 +191,38 @@ class ConfigDict(TypedDict, total=False):
     Whether an aliased field may be populated by its name as given by the model
     attribute, as well as the alias. Defaults to `False`.
 
-    Note:
-        The name of this configuration setting was changed in **v2.0** from
-        `allow_population_by_field_name` to `populate_by_name`.
+    !!! warning
+        `populate_by_name` usage is not recommended in v2.11+ and will be deprecated in v3.
+        Instead, you should use the [`validate_by_name`][pydantic.config.ConfigDict.validate_by_name] configuration setting.
 
-    ```python
-    from pydantic import BaseModel, ConfigDict, Field
+        When `validate_by_name=True` and `validate_by_alias=True`, this is strictly equivalent to the
+        previous behavior of `populate_by_name=True`.
 
-    class User(BaseModel):
-        model_config = ConfigDict(populate_by_name=True)
+        In v2.11, we also introduced a [`validate_by_alias`][pydantic.config.ConfigDict.validate_by_alias] setting that introduces more fine grained
+        control for validation behavior.
 
-        name: str = Field(alias='full_name')  # (1)!
-        age: int
+        Here's how you might go about using the new settings to achieve the same behavior:
 
-    user = User(full_name='John Doe', age=20)  # (2)!
-    print(user)
-    #> name='John Doe' age=20
-    user = User(name='John Doe', age=20)  # (3)!
-    print(user)
-    #> name='John Doe' age=20
-    ```
+        ```python
+        from pydantic import BaseModel, ConfigDict, Field
 
-    1. The field `'name'` has an alias `'full_name'`.
-    2. The model is populated by the alias `'full_name'`.
-    3. The model is populated by the field name `'name'`.
+        class Model(BaseModel):
+            model_config = ConfigDict(validate_by_name=True, validate_by_alias=True)
+
+            my_field: str = Field(alias='my_alias')  # (1)!
+
+        m = Model(my_alias='foo')  # (2)!
+        print(m)
+        #> my_field='foo'
+
+        m = Model(my_field='foo')  # (3)!
+        print(m)
+        #> my_field='foo'
+        ```
+
+        1. The field `'my_field'` has an alias `'my_alias'`.
+        2. The model is populated by the alias `'my_alias'`.
+        3. The model is populated by the attribute name `'my_field'`.
     """
 
     use_enum_values: bool
@@ -327,7 +388,7 @@ class ConfigDict(TypedDict, total=False):
     If you want to use different alias generators for validation and serialization, you can use
     [`AliasGenerator`][pydantic.aliases.AliasGenerator] instead.
 
-    If data source field names do not match your code style (e. g. CamelCase fields),
+    If data source field names do not match your code style (e.g. CamelCase fields),
     you can automatically generate aliases using `alias_generator`. Here's an example with
     a basic callable:
 
@@ -394,16 +455,16 @@ class ConfigDict(TypedDict, total=False):
     """
     A `dict` of custom JSON encoders for specific types. Defaults to `None`.
 
-    !!! warning "Deprecated"
-        This config option is a carryover from v1.
-        We originally planned to remove it in v2 but didn't have a 1:1 replacement so we are keeping it for now.
-        It is still deprecated and will likely be removed in the future.
+    /// version-deprecated | v2
+    This configuration option is a carryover from v1. We originally planned to remove it in v2 but didn't have a 1:1 replacement
+    so we are keeping it for now. It is still deprecated and will likely be removed in the future.
+    ///
     """
 
     # new in V2
     strict: bool
     """
-    _(new in V2)_ If `True`, strict validation is applied to all fields on the model.
+    Whether strict validation is applied to all fields on the model.
 
     By default, Pydantic attempts to coerce values to the correct type, when possible.
 
@@ -426,133 +487,81 @@ class ConfigDict(TypedDict, total=False):
 
     See the [Conversion Table](../concepts/conversion_table.md) for more details on how Pydantic converts data in both
     strict and lax modes.
+
+    /// version-added | v2
+    ///
     """
     # whether instances of models and dataclasses (including subclass instances) should re-validate, default 'never'
     revalidate_instances: Literal['always', 'never', 'subclass-instances']
     """
-    When and how to revalidate models and dataclasses during validation. Accepts the string
-    values of `'never'`, `'always'` and `'subclass-instances'`. Defaults to `'never'`.
+    When and how to revalidate models and dataclasses during validation. Can be one of:
 
-    - `'never'` will not revalidate models and dataclasses during validation
-    - `'always'` will revalidate models and dataclasses during validation
-    - `'subclass-instances'` will revalidate models and dataclasses during validation if the instance is a
+    - `'never'`: will *not* revalidate models and dataclasses during validation
+    - `'always'`: will revalidate models and dataclasses during validation
+    - `'subclass-instances'`: will revalidate models and dataclasses during validation if the instance is a
         subclass of the model or dataclass
 
-    By default, model and dataclass instances are not revalidated during validation.
+    The default is `'never'` (no revalidation).
+
+    This configuration only affects *the current model* it is applied on, and does *not* populate to the models
+    referenced in fields.
 
     ```python
-    from typing import List
-
     from pydantic import BaseModel
 
     class User(BaseModel, revalidate_instances='never'):  # (1)!
-        hobbies: List[str]
-
-    class SubUser(User):
-        sins: List[str]
+        name: str
 
     class Transaction(BaseModel):
         user: User
 
-    my_user = User(hobbies=['reading'])
+    my_user = User(name='John')
     t = Transaction(user=my_user)
-    print(t)
-    #> user=User(hobbies=['reading'])
 
-    my_user.hobbies = [1]  # (2)!
+    my_user.name = 1  # (2)!
     t = Transaction(user=my_user)  # (3)!
     print(t)
-    #> user=User(hobbies=[1])
-
-    my_sub_user = SubUser(hobbies=['scuba diving'], sins=['lying'])
-    t = Transaction(user=my_sub_user)
-    print(t)
-    #> user=SubUser(hobbies=['scuba diving'], sins=['lying'])
+    #> user=User(name=1)
     ```
 
-    1. `revalidate_instances` is set to `'never'` by **default.
-    2. The assignment is not validated, unless you set `validate_assignment` to `True` in the model's config.
-    3. Since `revalidate_instances` is set to `never`, this is not revalidated.
+    1. This is the default behavior.
+    2. The assignment is *not* validated, unless you set [`validate_assignment`][pydantic.ConfigDict.validate_assignment] in the configuration.
+    3. Since `revalidate_instances` is set to `'never'`, the user instance is not revalidated.
 
-    If you want to revalidate instances during validation, you can set `revalidate_instances` to `'always'`
-    in the model's config.
+    Here is an example demonstrating the behavior of `'subclass-instances'`:
 
     ```python
-    from typing import List
-
-    from pydantic import BaseModel, ValidationError
-
-    class User(BaseModel, revalidate_instances='always'):  # (1)!
-        hobbies: List[str]
-
-    class SubUser(User):
-        sins: List[str]
-
-    class Transaction(BaseModel):
-        user: User
-
-    my_user = User(hobbies=['reading'])
-    t = Transaction(user=my_user)
-    print(t)
-    #> user=User(hobbies=['reading'])
-
-    my_user.hobbies = [1]
-    try:
-        t = Transaction(user=my_user)  # (2)!
-    except ValidationError as e:
-        print(e)
-        '''
-        1 validation error for Transaction
-        user.hobbies.0
-          Input should be a valid string [type=string_type, input_value=1, input_type=int]
-        '''
-
-    my_sub_user = SubUser(hobbies=['scuba diving'], sins=['lying'])
-    t = Transaction(user=my_sub_user)
-    print(t)  # (3)!
-    #> user=User(hobbies=['scuba diving'])
-    ```
-
-    1. `revalidate_instances` is set to `'always'`.
-    2. The model is revalidated, since `revalidate_instances` is set to `'always'`.
-    3. Using `'never'` we would have gotten `user=SubUser(hobbies=['scuba diving'], sins=['lying'])`.
-
-    It's also possible to set `revalidate_instances` to `'subclass-instances'` to only revalidate instances
-    of subclasses of the model.
-
-    ```python
-    from typing import List
-
     from pydantic import BaseModel
 
-    class User(BaseModel, revalidate_instances='subclass-instances'):  # (1)!
-        hobbies: List[str]
+    class User(BaseModel, revalidate_instances='subclass-instances'):
+        name: str
 
     class SubUser(User):
-        sins: List[str]
+        age: int
 
     class Transaction(BaseModel):
         user: User
 
-    my_user = User(hobbies=['reading'])
-    t = Transaction(user=my_user)
-    print(t)
-    #> user=User(hobbies=['reading'])
-
-    my_user.hobbies = [1]
+    my_user = User(name='John')
+    my_user.name = 1  # (1)!
     t = Transaction(user=my_user)  # (2)!
     print(t)
-    #> user=User(hobbies=[1])
+    #> user=User(name=1)
 
-    my_sub_user = SubUser(hobbies=['scuba diving'], sins=['lying'])
+    my_sub_user = SubUser(name='John', age=20)
     t = Transaction(user=my_sub_user)
     print(t)  # (3)!
-    #> user=User(hobbies=['scuba diving'])
+    #> user=User(name='John')
     ```
 
-    1. `revalidate_instances` is set to `'subclass-instances'`.
-    2. This is not revalidated, since `my_user` is not a subclass of `User`.
-    3. Using `'never'` we would have gotten `user=SubUser(hobbies=['scuba diving'], sins=['lying'])`.
+    1. The assignment is *not* validated, unless you set [`validate_assignment`][pydantic.ConfigDict.validate_assignment] in the configuration.
+    2. Because `my_user` is a "direct" instance of `User`, it is *not* being revalidated. It would have been the case if
+      `revalidate_instances` was set to `'always'`.
+    3. Because `my_sub_user` is an instance of a `User` subclass, it is being revalidated. In this case, Pydantic coerces `my_sub_user` to the defined
+       `User` class defined on `Transaction`. If one of its fields had an invalid value, a validation error would have been raised.
+
+    /// version-added | v2
+    ///
     """
 
     ser_json_timedelta: Literal['iso8601', 'float']
@@ -560,8 +569,55 @@ class ConfigDict(TypedDict, total=False):
     The format of JSON serialized timedeltas. Accepts the string values of `'iso8601'` and
     `'float'`. Defaults to `'iso8601'`.
 
-    - `'iso8601'` will serialize timedeltas to ISO 8601 durations.
+    - `'iso8601'` will serialize timedeltas to [ISO 8601 text format](https://en.wikipedia.org/wiki/ISO_8601#Durations).
     - `'float'` will serialize timedeltas to the total number of seconds.
+
+    /// version-changed | v2.12
+    It is now recommended to use the [`ser_json_temporal`][pydantic.config.ConfigDict.ser_json_temporal]
+    setting. `ser_json_timedelta` will be deprecated in v3.
+    ///
+    """
+
+    ser_json_temporal: Literal['iso8601', 'seconds', 'milliseconds']
+    """
+    The format of JSON serialized temporal types from the [`datetime`][] module. This includes:
+
+    - [`datetime.datetime`][]
+    - [`datetime.date`][]
+    - [`datetime.time`][]
+    - [`datetime.timedelta`][]
+
+    Can be one of:
+
+    - `'iso8601'` will serialize date-like types to [ISO 8601 text format](https://en.wikipedia.org/wiki/ISO_8601#Durations).
+    - `'milliseconds'` will serialize date-like types to a floating point number of milliseconds since the epoch.
+    - `'seconds'` will serialize date-like types to a floating point number of seconds since the epoch.
+
+    Defaults to `'iso8601'`.
+
+    /// version-added | v2.12
+    This setting replaces [`ser_json_timedelta`][pydantic.config.ConfigDict.ser_json_timedelta],
+    which will be deprecated in v3. `ser_json_temporal` adds more configurability for the other temporal types.
+    ///
+    """
+
+    val_temporal_unit: Literal['seconds', 'milliseconds', 'infer']
+    """
+    The unit to assume for validating numeric input for datetime-like types ([`datetime.datetime`][] and [`datetime.date`][]). Can be one of:
+
+    - `'seconds'` will validate date or time numeric inputs as seconds since the [epoch].
+    - `'milliseconds'` will validate date or time numeric inputs as milliseconds since the [epoch].
+    - `'infer'` will infer the unit from the string numeric input on unix time as:
+
+        * seconds since the [epoch] if $-2^{10} <= v <= 2^{10}$
+        * milliseconds since the [epoch] (if $v < -2^{10}$ or $v > 2^{10}$).
+
+    Defaults to `'infer'`.
+
+    /// version-added | v2.12
+    ///
+
+    [epoch]: https://en.wikipedia.org/wiki/Unix_time
     """
 
     ser_json_bytes: Literal['utf8', 'base64', 'hex']
@@ -602,20 +658,15 @@ class ConfigDict(TypedDict, total=False):
 
     protected_namespaces: tuple[str | Pattern[str], ...]
     """
-    A `tuple` of strings and/or patterns that prevent models from having fields with names that conflict with them.
-    For strings, we match on a prefix basis. Ex, if 'dog' is in the protected namespace, 'dog_name' will be protected.
-    For patterns, we match on the entire field name. Ex, if `re.compile(r'^dog$')` is in the protected namespace, 'dog' will be protected, but 'dog_name' will not be.
-    Defaults to `('model_validate', 'model_dump',)`.
+    A tuple of strings and/or regex patterns that prevent models from having fields with names that conflict with its existing members/methods.
 
-    The reason we've selected these is to prevent collisions with other validation / dumping formats
-    in the future - ex, `model_validate_{some_newly_supported_format}`.
+    Strings are matched on a prefix basis. For instance, with `'dog'`, having a field named `'dog_name'` will be disallowed.
 
-    Before v2.10, Pydantic used `('model_',)` as the default value for this setting to
-    prevent collisions between model attributes and `BaseModel`'s own methods. This was changed
-    in v2.10 given feedback that this restriction was limiting in AI and data science contexts,
-    where it is common to have fields with names like `model_id`, `model_input`, `model_output`, etc.
+    Regex patterns are matched on the entire field name. For instance, with the pattern `'^dog$'`, having a field named `'dog'` will be disallowed,
+    but `'dog_name'` will be accepted.
 
-    For more details, see https://github.com/pydantic/pydantic/issues/10315.
+    Defaults to `('model_validate', 'model_dump')`. This default is used to prevent collisions with the existing (and possibly future)
+    [validation](../concepts/models.md#validating-data) and [serialization](../concepts/serialization.md#serializing-data) methods.
 
     ```python
     import warnings
@@ -632,9 +683,9 @@ class ConfigDict(TypedDict, total=False):
     except UserWarning as e:
         print(e)
         '''
-        Field "model_dump_something" in Model has conflict with protected namespace "model_dump".
+        Field 'model_dump_something' in 'Model' conflicts with protected namespace 'model_dump'.
 
-        You may be able to resolve this warning by setting `model_config['protected_namespaces'] = ('model_validate',)`.
+        You may be able to solve this by setting the 'protected_namespaces' configuration to ('model_validate',).
         '''
     ```
 
@@ -665,11 +716,11 @@ class ConfigDict(TypedDict, total=False):
     for warning in caught_warnings:
         print(f'{warning.message}')
         '''
-        Field "also_protect_field" in Model has conflict with protected namespace "also_protect_".
-        You may be able to resolve this warning by setting `model_config['protected_namespaces'] = ('protect_me_', re.compile('^protect_this$'))`.
+        Field 'also_protect_field' in 'Model' conflicts with protected namespace 'also_protect_'.
+        You may be able to solve this by setting the 'protected_namespaces' configuration to ('protect_me_', re.compile('^protect_this$'))`.
 
-        Field "protect_this" in Model has conflict with protected namespace "re.compile('^protect_this$')".
-        You may be able to resolve this warning by setting `model_config['protected_namespaces'] = ('protect_me_', 'also_protect_')`.
+        Field 'protect_this' in 'Model' conflicts with protected namespace 're.compile('^protect_this$')'.
+        You may be able to solve this by setting the 'protected_namespaces' configuration to ('protect_me_', 'also_protect_')`.
         '''
     ```
 
@@ -686,12 +737,17 @@ class ConfigDict(TypedDict, total=False):
 
             model_config = ConfigDict(protected_namespaces=('model_',))
 
-    except NameError as e:
+    except ValueError as e:
         print(e)
         '''
-        Field "model_validate" conflicts with member <bound method BaseModel.model_validate of <class 'pydantic.main.BaseModel'>> of protected namespace "model_".
+        Field 'model_validate' conflicts with member <bound method BaseModel.model_validate of <class 'pydantic.main.BaseModel'>> of protected namespace 'model_'.
         '''
     ```
+
+    /// version-changed | v2.10
+    The default protected namespaces was changed from `('model_',)` to `('model_validate', 'model_dump')`, to allow
+    for fields like `model_id`, `model_name` to be used.
+    ///
     """
 
     hide_input_in_errors: bool
@@ -746,7 +802,9 @@ class ConfigDict(TypedDict, total=False):
     used nested within other models, or when you want to manually define type namespace via
     [`Model.model_rebuild(_types_namespace=...)`][pydantic.BaseModel.model_rebuild].
 
-    Since v2.10, this setting also applies to pydantic dataclasses and TypeAdapter instances.
+    /// version-changed | v2.10
+    The setting also applies to [Pydantic dataclasses](../concepts/dataclasses.md) and [type adapters](../concepts/type_adapter.md).
+    ///
     """
 
     plugin_settings: dict[str, object] | None
@@ -754,12 +812,11 @@ class ConfigDict(TypedDict, total=False):
 
     schema_generator: type[_GenerateSchema] | None
     """
-    !!! warning
-        `schema_generator` is deprecated in v2.10.
+    The `GenerateSchema` class to use during core schema generation.
 
-        Prior to v2.10, this setting was advertised as highly subject to change.
-        It's possible that this interface may once again become public once the internal core schema generation
-        API is more stable, but that will likely come after significant performance improvements have been made.
+    /// version-deprecated | v2.10
+    The `GenerateSchema` class is private and highly subject to change.
+    ///
     """
 
     json_schema_serialization_defaults_required: bool
@@ -799,6 +856,9 @@ class ConfigDict(TypedDict, total=False):
     }
     '''
     ```
+
+    /// version-added | v2.4
+    ///
     """
 
     json_schema_mode_override: Literal['validation', 'serialization', None]
@@ -854,6 +914,9 @@ class ConfigDict(TypedDict, total=False):
     }
     '''
     ```
+
+    /// version-added | v2.4
+    ///
     """
 
     coerce_numbers_to_str: bool
@@ -899,14 +962,13 @@ class ConfigDict(TypedDict, total=False):
     The regex engine to be used for pattern validation.
     Defaults to `'rust-regex'`.
 
-    - `rust-regex` uses the [`regex`](https://docs.rs/regex) Rust crate,
+    - `'rust-regex'` uses the [`regex`](https://docs.rs/regex) Rust crate,
       which is non-backtracking and therefore more DDoS resistant, but does not support all regex features.
-    - `python-re` use the [`re`](https://docs.python.org/3/library/re.html) module,
-      which supports all regex features, but may be slower.
+    - `'python-re'` use the [`re`][] module, which supports all regex features, but may be slower.
 
     !!! note
-        If you use a compiled regex pattern, the python-re engine will be used regardless of this setting.
-        This is so that flags such as `re.IGNORECASE` are respected.
+        If you use a compiled regex pattern, the `'python-re'` engine will be used regardless of this setting.
+        This is so that flags such as [`re.IGNORECASE`][] are respected.
 
     ```python
     from pydantic import BaseModel, ConfigDict, Field, ValidationError
@@ -929,6 +991,9 @@ class ConfigDict(TypedDict, total=False):
           String should match pattern '^abc(?=def)' [type=string_pattern_mismatch, input_value='abxyzcdef', input_type=str]
         '''
     ```
+
+    /// version-added | v2.5
+    ///
     """
 
     validation_error_cause: bool
@@ -940,6 +1005,9 @@ class ConfigDict(TypedDict, total=False):
 
     Note:
         The structure of validation errors are likely to change in future Pydantic versions. Pydantic offers no guarantees about their structure. Should be used for visual traceback debugging only.
+
+    /// version-added | v2.5
+    ///
     """
 
     use_attribute_docstrings: bool
@@ -947,8 +1015,6 @@ class ConfigDict(TypedDict, total=False):
     Whether docstrings of attributes (bare string literals immediately following the attribute declaration)
     should be used for field descriptions. Defaults to `False`.
 
-    Available in Pydantic v2.7+.
-
     ```python
     from pydantic import BaseModel, ConfigDict, Field
 
@@ -974,10 +1040,15 @@ class ConfigDict(TypedDict, total=False):
     ```
     This requires the source code of the class to be available at runtime.
 
-    !!! warning "Usage with `TypedDict`"
-        Due to current limitations, attribute docstrings detection may not work as expected when using `TypedDict`
-        (in particular when multiple `TypedDict` classes have the same name in the same source file). The behavior
-        can be different depending on the Python version used.
+    !!! warning "Usage with `TypedDict` and stdlib dataclasses"
+        Due to current limitations, attribute docstrings detection may not work as expected when using
+        [`TypedDict`][typing.TypedDict] and stdlib dataclasses, in particular when:
+
+        - inheritance is being used.
+        - multiple classes have the same name in the same source file (unless Python 3.13 or greater is used).
+
+    /// version-added | v2.7
+    ///
     '''
 
     cache_strings: bool | Literal['all', 'keys', 'none']
@@ -997,14 +1068,162 @@ class ConfigDict(TypedDict, total=False):
     !!! tip
         If repeated strings are rare, it's recommended to use `'keys'` or `'none'` to reduce memory usage,
         as the performance difference is minimal if repeated strings are rare.
+
+    /// version-added | v2.7
+    ///
+    """
+
+    validate_by_alias: bool
+    """
+    Whether an aliased field may be populated by its alias. Defaults to `True`.
+
+    Here's an example of disabling validation by alias:
+
+    ```py
+    from pydantic import BaseModel, ConfigDict, Field
+
+    class Model(BaseModel):
+        model_config = ConfigDict(validate_by_name=True, validate_by_alias=False)
+
+        my_field: str = Field(validation_alias='my_alias')  # (1)!
+
+    m = Model(my_field='foo')  # (2)!
+    print(m)
+    #> my_field='foo'
+    ```
+
+    1. The field `'my_field'` has an alias `'my_alias'`.
+    2. The model can only be populated by the attribute name `'my_field'`.
+
+    !!! warning
+        You cannot set both `validate_by_alias` and `validate_by_name` to `False`.
+        This would make it impossible to populate an attribute.
+
+        See [usage errors](../errors/usage_errors.md#validate-by-alias-and-name-false) for an example.
+
+        If you set `validate_by_alias` to `False`, under the hood, Pydantic dynamically sets
+        `validate_by_name` to `True` to ensure that validation can still occur.
+
+    /// version-added | v2.11
+    This setting was introduced in conjunction with [`validate_by_name`][pydantic.ConfigDict.validate_by_name]
+    to empower users with more fine grained validation control.
+    ///
+    """
+
+    validate_by_name: bool
+    """
+    Whether an aliased field may be populated by its name as given by the model
+    attribute. Defaults to `False`.
+
+    ```python
+    from pydantic import BaseModel, ConfigDict, Field
+
+    class Model(BaseModel):
+        model_config = ConfigDict(validate_by_name=True, validate_by_alias=True)
+
+        my_field: str = Field(validation_alias='my_alias')  # (1)!
+
+    m = Model(my_alias='foo')  # (2)!
+    print(m)
+    #> my_field='foo'
+
+    m = Model(my_field='foo')  # (3)!
+    print(m)
+    #> my_field='foo'
+    ```
+
+    1. The field `'my_field'` has an alias `'my_alias'`.
+    2. The model is populated by the alias `'my_alias'`.
+    3. The model is populated by the attribute name `'my_field'`.
+
+    !!! warning
+        You cannot set both `validate_by_alias` and `validate_by_name` to `False`.
+        This would make it impossible to populate an attribute.
+
+        See [usage errors](../errors/usage_errors.md#validate-by-alias-and-name-false) for an example.
+
+    /// version-added | v2.11
+    This setting was introduced in conjunction with [`validate_by_alias`][pydantic.ConfigDict.validate_by_alias]
+    to empower users with more fine grained validation control. It is an alternative to [`populate_by_name`][pydantic.ConfigDict.populate_by_name],
+    that enables validation by name **and** by alias.
+    ///
+    """
+
+    serialize_by_alias: bool
+    """
+    Whether an aliased field should be serialized by its alias. Defaults to `False`.
+
+    Note: In v2.11, `serialize_by_alias` was introduced to address the
+    [popular request](https://github.com/pydantic/pydantic/issues/8379)
+    for consistency with alias behavior for validation and serialization settings.
+    In v3, the default value is expected to change to `True` for consistency with the validation default.
+
+    ```python
+    from pydantic import BaseModel, ConfigDict, Field
+
+    class Model(BaseModel):
+        model_config = ConfigDict(serialize_by_alias=True)
+
+        my_field: str = Field(serialization_alias='my_alias')  # (1)!
+
+    m = Model(my_field='foo')
+    print(m.model_dump())  # (2)!
+    #> {'my_alias': 'foo'}
+    ```
+
+    1. The field `'my_field'` has an alias `'my_alias'`.
+    2. The model is serialized using the alias `'my_alias'` for the `'my_field'` attribute.
+
+
+    /// version-added | v2.11
+    This setting was introduced to address the [popular request](https://github.com/pydantic/pydantic/issues/8379)
+    for consistency with alias behavior for validation and serialization.
+
+    In v3, the default value is expected to change to `True` for consistency with the validation default.
+    ///
+    """
+
+    url_preserve_empty_path: bool
+    """
+    Whether to preserve empty URL paths when validating values for a URL type. Defaults to `False`.
+
+    ```python
+    from pydantic import AnyUrl, BaseModel, ConfigDict
+
+    class Model(BaseModel):
+        model_config = ConfigDict(url_preserve_empty_path=True)
+
+        url: AnyUrl
+
+    m = Model(url='http://example.com')
+    print(m.url)
+    #> http://example.com
+    ```
+
+    /// version-added | v2.12
+    ///
     """
 
 
 _TypeT = TypeVar('_TypeT', bound=type)
 
 
-def with_config(config: ConfigDict) -> Callable[[_TypeT], _TypeT]:
-    """Usage docs: https://docs.pydantic.dev/2.10/concepts/config/#configuration-with-dataclass-from-the-standard-library-or-typeddict
+@overload
+@deprecated('Passing `config` as a keyword argument is deprecated. Pass `config` as a positional argument instead.')
+def with_config(*, config: ConfigDict) -> Callable[[_TypeT], _TypeT]: ...
+
+
+@overload
+def with_config(config: ConfigDict, /) -> Callable[[_TypeT], _TypeT]: ...
+
+
+@overload
+def with_config(**config: Unpack[ConfigDict]) -> Callable[[_TypeT], _TypeT]: ...
+
+
+def with_config(config: ConfigDict | None = None, /, **kwargs: Any) -> Callable[[_TypeT], _TypeT]:
+    """!!! abstract "Usage Documentation"
+        [Configuration with other types](../concepts/config.md#configuration-on-other-supported-types)
 
     A convenience decorator to set a [Pydantic configuration](config.md) on a `TypedDict` or a `dataclass` from the standard library.
 
@@ -1019,15 +1238,35 @@ def with_config(config: ConfigDict) -> Callable[[_TypeT], _TypeT]:
         from pydantic import ConfigDict, TypeAdapter, with_config
 
         @with_config(ConfigDict(str_to_lower=True))
-        class Model(TypedDict):
+        class TD(TypedDict):
             x: str
 
-        ta = TypeAdapter(Model)
+        ta = TypeAdapter(TD)
 
         print(ta.validate_python({'x': 'ABC'}))
         #> {'x': 'abc'}
         ```
+
+    /// deprecated-removed | v2.11 v3
+    Passing `config` as a keyword argument.
+    ///
+
+    /// version-changed | v2.11
+    Keyword arguments can be provided directly instead of a config dictionary.
+    ///
     """
+    if config is not None and kwargs:
+        raise ValueError('Cannot specify both `config` and keyword arguments')
+
+    if len(kwargs) == 1 and (kwargs_conf := kwargs.get('config')) is not None:
+        warnings.warn(
+            'Passing `config` as a keyword argument is deprecated. Pass `config` as a positional argument instead',
+            category=PydanticDeprecatedSince211,
+            stacklevel=2,
+        )
+        final_config = cast(ConfigDict, kwargs_conf)
+    else:
+        final_config = config if config is not None else cast(ConfigDict, kwargs)
 
     def inner(class_: _TypeT, /) -> _TypeT:
         # Ideally, we would check for `class_` to either be a `TypedDict` or a stdlib dataclass.
@@ -1040,7 +1279,7 @@ def with_config(config: ConfigDict) -> Callable[[_TypeT], _TypeT]:
                 f'Cannot use `with_config` on {class_.__name__} as it is a Pydantic model',
                 code='with-config-on-model',
             )
-        class_.__pydantic_config__ = config
+        class_.__pydantic_config__ = final_config
         return class_
 
     return inner
