@@ -1,4 +1,4 @@
-from typing import Generic, List, Optional, Set, TypeVar, Union
+from typing import Generic, Optional, Set, TypeVar, Union
 
 from pydantic import BaseModel, ConfigDict, Extra, Field, field_validator
 from pydantic.dataclasses import dataclass
@@ -153,14 +153,14 @@ class DynamicAliasModel2(BaseModel):
     x: str = Field(..., alias=x_alias)
     z: int
 
-    model_config = ConfigDict(populate_by_name=True)
+    model_config = ConfigDict(validate_by_name=True)
 
 
 DynamicAliasModel2(y='y', z=1)
 DynamicAliasModel2(x='y', z=1)
 
 
-class KwargsDynamicAliasModel(BaseModel, populate_by_name=True):
+class KwargsDynamicAliasModel(BaseModel, validate_by_name=True):
     x: str = Field(..., alias=x_alias)
     z: int
 
