@@ -1,15 +1,11 @@
-import sys
+from collections.abc import Callable as CollectionsCallable
 from typing import Callable
 
 import pytest
 
 from pydantic import BaseModel, ValidationError
 
-collection_callable_types = [Callable, Callable[[int], int]]
-if sys.version_info >= (3, 9):
-    from collections.abc import Callable as CollectionsCallable
-
-    collection_callable_types += [CollectionsCallable, CollectionsCallable[[int], int]]
+collection_callable_types = [Callable, Callable[[int], int], CollectionsCallable, CollectionsCallable[[int], int]]
 
 
 @pytest.mark.parametrize('annotation', collection_callable_types)
