@@ -7,6 +7,7 @@ import traceback
 from collections.abc import Hashable
 from dataclasses import InitVar
 from datetime import date, datetime
+from functools import cached_property
 from pathlib import Path
 from typing import (
     Annotated,
@@ -30,6 +31,7 @@ from pydantic import (
     BaseModel,
     BeforeValidator,
     ConfigDict,
+    Field,
     PydanticDeprecatedSince20,
     PydanticUndefinedAnnotation,
     PydanticUserError,
@@ -45,7 +47,6 @@ from pydantic import (
 )
 from pydantic._internal._mock_val_ser import MockValSer
 from pydantic.dataclasses import is_pydantic_dataclass, rebuild_dataclass
-from pydantic.fields import Field, FieldInfo
 from pydantic.json_schema import model_json_schema
 
 
@@ -176,6 +177,39 @@ def test_validate_assignment_value_change():
     assert d.a == 6
 
 
+def test_validate_assignment_properties() -> None:
+    """https://github.com/pydantic/pydantic/issues/12112"""
+
+    @pydantic.dataclasses.dataclass(config=ConfigDict(validate_assignment=True))
+    class MyDataclass:
+        @property
+        def prop1(self) -> int:
+            return 1
+
+        @prop1.setter
+        def prop1(self, value: int) -> None:
+            pass
+
+        @computed_field
+        @property
+        def prop2(self) -> int:
+            return 1
+
+        @prop2.setter
+        def prop2(self, value: int) -> None:
+            pass
+
+        @cached_property
+        def prop3(self) -> int:
+            return 1
+
+    m = MyDataclass()
+
+    m.prop1 = 1
+    m.prop2 = 1
+    m.prop3 = 1
+
+
 @pytest.mark.parametrize(
     'config',
     [
@@ -1093,7 +1127,9 @@ def test_dataclass_equality_for_field_values(foo_bar_getter):
     assert foo == bar.c
 
 
-def test_issue_2383():
+def test_hash_method_preserved() -> None:
+    """https://github.com/pydantic/pydantic/issues/2383"""
+
     @dataclasses.dataclass
     class A:
         s: str
@@ -1111,7 +1147,9 @@ def test_issue_2383():
     assert hash(b.a) == 123
 
 
-def test_issue_2398():
+def test_order_preserved() -> None:
+    """https://github.com/pydantic/pydantic/issues/2398"""
+
     @dataclasses.dataclass(order=True)
     class DC:
         num: int = 42
@@ -1128,7 +1166,9 @@ def test_issue_2398():
     assert real_dc <= model.dc
 
 
-def test_issue_2424():
+def test_default_factory_works_on_subclasses() -> None:
+    """https://github.com/pydantic/pydantic/issues/2424"""
+
     @dataclasses.dataclass
     class Base:
         x: str
@@ -1147,7 +1187,23 @@ def test_issue_2424():
     assert ValidatedThing(x='hi').y == ''
 
 
-def test_issue_2541():
+def test_override_default_stdlib_dataclass() -> None:
+    """https://github.com/pydantic/pydantic/issues/11816"""
+
+    @dataclasses.dataclass
+    class Test:
+        value: int = 1
+
+    @pydantic.dataclasses.dataclass
+    class Sub(Test):
+        value: int = 2
+
+    assert Sub().value == 2
+
+
+def test_frozen_preserved_on_model_field() -> None:
+    """https://github.com/pydantic/pydantic/issues/2541"""
+
     @dataclasses.dataclass(frozen=True)
     class Infos:
         id: int
@@ -1272,7 +1328,9 @@ def test_json_schema_with_computed_field():
     }
 
 
-def test_issue_2594():
+def test_supports_stdlib_dataclass_without_annotations() -> None:
+    """https://github.com/pydantic/pydantic/issues/2594"""
+
     @dataclasses.dataclass
     class Empty:
         pass
@@ -1318,8 +1376,8 @@ def test_schema_description_set():
     assert model_json_schema(A)['description'] == 'my description'
 
 
-def test_issue_3011():
-    """Validation of a subclass of a dataclass"""
+def test_subclass_of_a_dataclass_supported() -> None:
+    """https://github.com/pydantic/pydantic/issues/3011"""
 
     @dataclasses.dataclass
     class A:
@@ -1337,7 +1395,9 @@ def test_issue_3011():
     assert c.thing.thing_a == 'Thing A'
 
 
-def test_issue_3162():
+def test_dataclass_referenced_twice() -> None:
+    """https://github.com/pydantic/pydantic/issues/3162"""
+
     @dataclasses.dataclass
     class User:
         id: int
@@ -1815,6 +1875,41 @@ def test_kw_only_inheritance(decorator1, decorator2):
     assert child.y == 1
 
 
+def test_kw_only_inheritance_on_field() -> None:
+    @dataclasses.dataclass
+    class A:
+        x: int = Field(kw_only=True)
+
+    @pydantic.dataclasses.dataclass
+    class B(A):
+        pass
+
+    if sys.version_info >= (3, 10):  # On 3.9, we ignore kw_only.
+        with pytest.raises(ValidationError):
+            B(1)
+
+
+def test_repr_inheritance() -> None:
+    @dataclasses.dataclass
+    class A:
+        a: int = Field(repr=False)
+
+    @pydantic.dataclasses.dataclass
+    class B(A):
+        pass
+
+    assert repr(B(a=1)).endswith('B()')
+
+
+@pytest.mark.skipif(sys.version_info < (3, 14), reason='`doc` added in 3.14')
+def test_description_as_doc_in_slots() -> None:
+    @pydantic.dataclasses.dataclass(slots=True)
+    class A:
+        a: int = Field(description='a doc')
+
+    assert A.__slots__ == {'a': 'a doc'}
+
+
 def test_extra_forbid_list_no_error():
     @pydantic.dataclasses.dataclass(config=dict(extra='forbid'))
     class Bar: ...
@@ -2072,15 +2167,14 @@ def test_inheritance_replace(decorator1: Callable[[Any], Any], expected_parent:
 def test_dataclasses_inheritance_default_value_is_not_deleted(
     decorator1: Callable[[Any], Any], default: Literal[1]
 ) -> None:
-    if decorator1 is dataclasses.dataclass and isinstance(default, FieldInfo):
-        pytest.skip(reason="stdlib dataclasses don't support Pydantic fields")
-
     @decorator1
     class Parent:
         a: int = default
 
-    assert Parent.a == 1
-    assert Parent().a == 1
+    # stdlib dataclasses don't support Pydantic's `Field()`:
+    if decorator1 is pydantic.dataclasses.dataclass:
+        assert Parent.a == 1
+        assert Parent().a == 1
 
     @pydantic.dataclasses.dataclass
     class Child(Parent):
@@ -2090,6 +2184,37 @@ def test_dataclasses_inheritance_default_value_is_not_deleted(
     assert Child().a == 1
 
 
+def test_dataclasses_inheritance_bare_class_not_used() -> None:
+    """https://github.com/pydantic/pydantic/issues/12045"""
+
+    class BareClass:
+        a: int = Field(kw_only=True)
+
+    @pydantic.dataclasses.dataclass
+    class DC(BareClass):
+        pass
+
+    assert len(DC.__dataclass_fields__) == 0
+    assert len(DC.__pydantic_fields__) == 0
+
+
+def test_dataclasses_type_override_pydantic_field() -> None:
+    """https://github.com/pydantic/pydantic/issues/12045.
+
+    `B.a` used to be typed as `str`, only if `pydantic.Field()` was being used on `A.a`.
+    """
+
+    @dataclasses.dataclass
+    class A:
+        a: int = Field()
+
+    @pydantic.dataclasses.dataclass
+    class B(A):
+        a: str = dataclasses.field()
+
+    assert B(a='test').a == 'test'
+
+
 def test_dataclass_config_validate_default():
     @pydantic.dataclasses.dataclass
     class Model:
@@ -2430,6 +2555,23 @@ def test_dataclass_slots(dataclass_decorator):
     assert dc.b == 'bar'
 
 
+# Must be defined at the module level to be picklable:
+@pydantic.dataclasses.dataclass(slots=True, config={'validate_assignment': True})
+class DataclassSlotsValidateAssignment:
+    a: int
+
+
+@pytest.mark.skipif(sys.version_info < (3, 10), reason='slots are only supported for dataclasses in Python >= 3.10')
+def test_dataclass_slots_validate_assignment():
+    """https://github.com/pydantic/pydantic/issues/11768"""
+
+    m = DataclassSlotsValidateAssignment(1)
+    m_pickle = pickle.loads(pickle.dumps(m))
+    assert m_pickle.a == 1
+    with pytest.raises(ValidationError):
+        m.a = 'not_an_int'
+
+
 @pytest.mark.parametrize(
     'dataclass_decorator',
     [
@@ -3003,10 +3145,8 @@ def test_simple_frozen() -> None:
 
     inst = MyDataclass('hello')
 
-    try:
+    with pytest.raises(dataclasses.FrozenInstanceError, match="cannot assign to field 'x'"):
         inst.x = 'other'
-    except Exception as e:
-        assert "cannot assign to field 'x'" in repr(e)
 
     @pydantic.dataclasses.dataclass(config=ConfigDict(frozen=True))
     class MyDataclass2:
@@ -3014,10 +3154,8 @@ def test_simple_frozen() -> None:
 
     inst = MyDataclass2('hello')
 
-    try:
+    with pytest.raises(dataclasses.FrozenInstanceError, match="cannot assign to field 'x'"):
         inst.x = 'other'
-    except Exception as e:
-        assert "cannot assign to field 'x'" in repr(e)
 
 
 def test_frozen_with_validate_assignment() -> None:
@@ -3029,10 +3167,8 @@ def test_frozen_with_validate_assignment() -> None:
 
     inst = MyDataclass('hello')
 
-    try:
+    with pytest.raises(dataclasses.FrozenInstanceError, match="cannot assign to field 'x'"):
         inst.x = 'other'
-    except Exception as e:
-        assert "cannot assign to field 'x'" in repr(e)
 
     @pydantic.dataclasses.dataclass(config=ConfigDict(frozen=True, validate_assignment=True))
     class MyDataclass2:
@@ -3040,11 +3176,8 @@ def test_frozen_with_validate_assignment() -> None:
 
     inst = MyDataclass2('hello')
 
-    # we want to make sure that the error raised relates to the frozen nature of the instance
-    try:
+    with pytest.raises(dataclasses.FrozenInstanceError, match="cannot assign to field 'x'"):
         inst.x = 'other'
-    except ValidationError as e:
-        assert 'Instance is frozen' in repr(e)
 
 
 def test_warns_on_double_frozen() -> None:
@@ -3106,3 +3239,20 @@ def test_dataclass_fields_rebuilt_before_schema_generation() -> None:
     ta = TypeAdapter(A)
 
     assert ta.json_schema()['properties']['a']['test'] == 'a'
+
+
+def test_dataclass_field_exclude() -> None:
+    @pydantic.dataclasses.dataclass
+    class Foo:
+        foo: str = Field(exclude=True)
+        bar: int = Field(exclude_if=lambda x: x > 1)
+
+    ta = TypeAdapter(Foo)
+
+    assert ta.dump_python(Foo(foo='bar', bar=1)) == {'bar': 1}
+    assert ta.dump_python(Foo(foo='bar', bar=1), exclude={'bar'}) == {}
+    assert ta.dump_python(Foo(foo='bar', bar=2)) == {}
+
+    assert ta.dump_json(Foo(foo='bar', bar=1)).decode('utf-8') == '{"bar":1}'
+    assert ta.dump_json(Foo(foo='bar', bar=1), exclude={'bar'}).decode('utf-8') == '{}'
+    assert ta.dump_json(Foo(foo='bar', bar=2)).decode('utf-8') == '{}'
