@@ -1,17 +1,20 @@
 import asyncio
 import inspect
 from pathlib import Path
-from typing import List
+from typing import Annotated
 
 import pytest
 from dirty_equals import IsInstance
-from typing_extensions import Annotated
 
 from pydantic import BaseModel, Field, PydanticDeprecatedSince20, ValidationError
 from pydantic.deprecated.decorator import ValidatedFunction
 from pydantic.deprecated.decorator import validate_arguments as validate_arguments_deprecated
 from pydantic.errors import PydanticUserError
 
+# `pytest.warns/raises()` is thread unsafe. As these tests are meant to be
+# removed in V3, we just mark all tests as thread unsafe
+pytestmark = pytest.mark.thread_unsafe
+
 
 def validate_arguments(*args, **kwargs):
     with pytest.warns(
@@ -43,7 +46,7 @@ def test_args():
         {
             'input': 'x',
             'loc': ('b',),
-            'msg': 'Input should be a valid integer, unable to parse string as an ' 'integer',
+            'msg': 'Input should be a valid integer, unable to parse string as an integer',
             'type': 'int_parsing',
         }
     ]
@@ -97,7 +100,7 @@ def test_kwargs():
         {
             'input': 'x',
             'loc': ('b',),
-            'msg': 'Input should be a valid integer, unable to parse string as an ' 'integer',
+            'msg': 'Input should be a valid integer, unable to parse string as an integer',
             'type': 'int_parsing',
         }
     ]
@@ -239,7 +242,7 @@ def test_async():
 
 def test_string_annotation():
     @validate_arguments
-    def foo(a: 'List[int]', b: 'Path'):
+    def foo(a: 'list[int]', b: 'Path'):
         return f'a={a!r} b={b!r}'
 
     assert foo([1, 2, 3], '/')
@@ -250,7 +253,7 @@ def test_string_annotation():
         {
             'input': 'x',
             'loc': ('a', 0),
-            'msg': 'Input should be a valid integer, unable to parse string as an ' 'integer',
+            'msg': 'Input should be a valid integer, unable to parse string as an integer',
             'type': 'int_parsing',
         },
         {'input': {'a': ['x']}, 'loc': ('b',), 'msg': 'Field required', 'type': 'missing'},
@@ -350,7 +353,7 @@ def test_config_arbitrary_types_allowed():
             'ctx': {'class': 'test_config_arbitrary_types_allowed.<locals>.EggBox'},
             'input': 2,
             'loc': ('b',),
-            'msg': 'Input should be an instance of ' 'test_config_arbitrary_types_allowed.<locals>.EggBox',
+            'msg': 'Input should be an instance of test_config_arbitrary_types_allowed.<locals>.EggBox',
             'type': 'is_instance_of',
         }
     ]
@@ -378,8 +381,8 @@ def test_use_of_alias():
     assert foo(b=10) == 30
 
 
-def test_populate_by_name():
-    @validate_arguments(config=dict(populate_by_name=True))
+def test_validate_by_name():
+    @validate_arguments(config=dict(validate_by_name=True, validate_by_alias=True))
     def foo(a: Annotated[int, Field(alias='b')], c: Annotated[int, Field(alias='d')]):
         return a + c
 
