@@ -1,5 +1,6 @@
+import sys
 import textwrap
-from dataclasses import dataclass
+from dataclasses import dataclass, field
 from typing import Annotated, Generic, TypeVar
 
 import pytest
@@ -173,7 +174,6 @@ def test_model_different_name():
         )
 
     MyModel.__name__ = 'OtherModel'
-    print(MyModel.__name__)
 
     assert MyModel.model_fields['a'].description == 'A docs'
 
@@ -230,30 +230,37 @@ def test_dataclass_docs_extraction():
         def dummy_method(self) -> None:
             """Docs for dummy_method that won't be used for d"""
 
-        e: int = Field(1, description='Real description')
+        e: int = Field(1, description='Real description e')
         """Won't be used"""
 
-        f: int = 1
-        """F docs"""
-
-        """Useless docs"""
+        if sys.version_info >= (3, 14):
+            f: int = field(default=1, doc='Real description f')
+            """Won't be used"""
 
         g: int = 1
         """G docs"""
 
-        h = 1
+        """Useless docs"""
+
+        h: int = 1
         """H docs"""
 
-        i: Annotated[int, Field(description='Real description')] = 1
+        i = 1
+        """I docs, not a field"""
+
+        j: Annotated[int, Field(description='Real description j')] = 1
         """Won't be used"""
 
     assert MyModel.__pydantic_fields__['a'].description == 'A docs'
     assert MyModel.__pydantic_fields__['b'].description == 'B docs'
     assert MyModel.__pydantic_fields__['c'].description is None
     assert MyModel.__pydantic_fields__['d'].description is None
-    assert MyModel.__pydantic_fields__['e'].description == 'Real description'
+    assert MyModel.__pydantic_fields__['e'].description == 'Real description e'
+    if sys.version_info >= (3, 14):
+        assert MyModel.__pydantic_fields__['f'].description == 'Real description f'
     assert MyModel.__pydantic_fields__['g'].description == 'G docs'
-    assert MyModel.__pydantic_fields__['i'].description == 'Real description'
+    assert MyModel.__pydantic_fields__['h'].description == 'H docs'
+    assert MyModel.__pydantic_fields__['j'].description == 'Real description j'
 
     # https://github.com/pydantic/pydantic/issues/11243:
     # Even though the `FieldInfo` instances had the correct description set,
@@ -270,12 +277,50 @@ def test_stdlib_docs_extraction():
 
     ta = TypeAdapter(MyModel)
 
-    assert ta.json_schema() == {
-        'properties': {'a': {'title': 'A', 'type': 'integer', 'description': 'A docs'}},
-        'required': ['a'],
-        'title': 'MyModel',
-        'type': 'object',
-    }
+    assert ta.json_schema()['properties']['a']['description'] == 'A docs'
+
+
+@pytest.mark.xfail(
+    condition=sys.version_info < (3, 13),
+    reason=(
+        'Since Python 3.13, we can leverage the new `__firstlineno__` class attribute, '
+        'used by `inspect.getsourcelines().'
+    ),
+)
+def test_stdlib_docs_extraction_duplicate_class():
+    @dataclass
+    @with_config({'use_attribute_docstrings': True})
+    class MyModel:
+        a: int
+        """A docs"""
+
+    @dataclass
+    @with_config({'use_attribute_docstrings': True})
+    class MyModel:
+        b: int
+        """B docs"""
+
+    ta = TypeAdapter(MyModel)
+    assert ta.json_schema()['properties']['b']['description'] == 'B docs'
+
+    if True:
+
+        @dataclass
+        @with_config({'use_attribute_docstrings': True})
+        class MyModel:
+            a: int
+            """A docs"""
+
+    else:
+
+        @dataclass
+        @with_config({'use_attribute_docstrings': True})
+        class MyModel:
+            b: int
+            """B docs"""
+
+    ta = TypeAdapter(MyModel)
+    assert ta.json_schema()['properties']['a']['description'] == 'A docs'
 
 
 @pytest.mark.xfail(reason='Current implementation does not take inheritance into account.')
