@@ -3,7 +3,6 @@
 from __future__ import annotations
 
 import datetime
-import warnings
 from decimal import Decimal
 from typing import Annotated, Any, Callable, Union
 
@@ -11,11 +10,8 @@ import pytest
 import pytz
 from annotated_types import Interval
 
-from pydantic import PydanticExperimentalWarning, TypeAdapter, ValidationError
-
-with warnings.catch_warnings():
-    warnings.filterwarnings('ignore', category=PydanticExperimentalWarning)
-    from pydantic.experimental.pipeline import _Pipeline, transform, validate_as  # type: ignore
+from pydantic import TypeAdapter, ValidationError
+from pydantic.experimental.pipeline import _Pipeline, transform, validate_as  # pyright: ignore[reportPrivateUsage]
 
 
 @pytest.mark.parametrize('potato_variation', ['potato', ' potato ', ' potato', 'potato ', ' POTATO ', ' PoTatO '])
@@ -181,6 +177,55 @@ def test_parse_tz() -> None:
         ta_tza.validate_python(date)
 
 
+def test_timezone_constraint_else_block() -> None:
+    """Test to hit the else block in Timezone constraint handling.
+
+    This happens when tz is ... but the schema is NOT datetime type.
+    The else block creates a check_tz_aware function that expects datetime objects.
+    """
+
+    ta_tz_aware = TypeAdapter[datetime.datetime](
+        Annotated[
+            datetime.datetime,
+            validate_as(str)
+            .transform(
+                lambda x: datetime.datetime.strptime(x, '%Y-%m-%dT%H:%M:%S.%f').replace(tzinfo=datetime.timezone.utc)
+            )
+            .datetime_tz_aware(),
+        ]
+    )
+
+    assert ta_tz_aware.validate_python('2032-06-04T11:15:30.400000') == datetime.datetime(
+        2032, 6, 4, 11, 15, 30, 400000, tzinfo=datetime.timezone.utc
+    )
+
+    ta_tz_aware_success = TypeAdapter[datetime.datetime](
+        Annotated[
+            datetime.datetime,
+            validate_as(str)
+            .transform(lambda x: datetime.datetime.strptime(x, '%Y-%m-%dT%H:%M:%S.%f%z'))
+            .datetime_tz_aware(),
+        ]
+    )
+
+    result = ta_tz_aware_success.validate_python('2032-06-04T11:15:30.400000+00:00')
+    assert result == datetime.datetime(2032, 6, 4, 11, 15, 30, 400000, tzinfo=datetime.timezone.utc)
+    assert result.tzinfo is not None
+
+    ta_tz_naive = TypeAdapter[datetime.datetime](
+        Annotated[
+            datetime.datetime,
+            validate_as(str)
+            .transform(lambda x: datetime.datetime.strptime(x, '%Y-%m-%dT%H:%M:%S.%f'))
+            .datetime_tz_naive(),
+        ]
+    )
+
+    result_naive = ta_tz_naive.validate_python('2032-06-04T11:15:30.400000')
+    assert result_naive.tzinfo is None
+    assert result_naive == datetime.datetime(2032, 6, 4, 11, 15, 30, 400000)
+
+
 @pytest.mark.parametrize(
     'method, method_arg, input_string, expected_output',
     [
@@ -255,6 +300,11 @@ def test_predicates() -> None:
     with pytest.raises(ValidationError):
         ta_str.validate_python('potato')
 
+    ta_str_to_int = TypeAdapter[int](
+        Annotated[str, validate_as(str).transform(lambda x: int(float(x))).predicate(float)]
+    )
+    assert ta_str_to_int.validate_python('1.5') == 1
+
 
 @pytest.mark.parametrize(
     'model, expected_val_schema, expected_ser_schema',
