@@ -1,10 +1,4 @@
-import sys
-from typing import Any, Type
-
-if sys.version_info < (3, 9):
-    from typing_extensions import Annotated
-else:
-    from typing import Annotated
+from typing import Annotated, Any
 
 import pytest
 
@@ -27,7 +21,7 @@ def model_with_strict_field():
         1.0,
     ],
 )
-def test_parse_strict_mode_on_field_invalid(value: Any, ModelWithStrictField: Type[BaseModel]) -> None:
+def test_parse_strict_mode_on_field_invalid(value: Any, ModelWithStrictField: type[BaseModel]) -> None:
     with pytest.raises(ValidationError) as exc_info:
         ModelWithStrictField(a=value)
     assert exc_info.value.errors(include_url=False) == [
@@ -35,7 +29,7 @@ def test_parse_strict_mode_on_field_invalid(value: Any, ModelWithStrictField: Ty
     ]
 
 
-def test_parse_strict_mode_on_field_valid(ModelWithStrictField: Type[BaseModel]) -> None:
+def test_parse_strict_mode_on_field_valid(ModelWithStrictField: type[BaseModel]) -> None:
     value = ModelWithStrictField(a=1)
     assert value.model_dump() == {'a': 1}
 
@@ -56,7 +50,7 @@ def model_with_strict_config_false():
     return ModelWithStrictConfig
 
 
-def test_parse_model_with_strict_config_enabled(ModelWithStrictConfig: Type[BaseModel]) -> None:
+def test_parse_model_with_strict_config_enabled(ModelWithStrictConfig: type[BaseModel]) -> None:
     with pytest.raises(ValidationError) as exc_info:
         ModelWithStrictConfig(a='1', b=2, c=3, d=4)
     assert exc_info.value.errors(include_url=False) == [
@@ -79,7 +73,7 @@ def test_parse_model_with_strict_config_enabled(ModelWithStrictConfig: Type[Base
     assert all(v.model_dump() == {'a': 1, 'b': 2, 'c': 3, 'd': 4} for v in values)
 
 
-def test_parse_model_with_strict_config_disabled(ModelWithStrictConfig: Type[BaseModel]) -> None:
+def test_parse_model_with_strict_config_disabled(ModelWithStrictConfig: type[BaseModel]) -> None:
     class Model(ModelWithStrictConfig):
         model_config = ConfigDict(strict=False)
 
