@@ -1,4 +1,5 @@
-from typing import Dict, List, Mapping, Union
+from collections.abc import Mapping
+from typing import Union
 
 import pytest
 
@@ -9,7 +10,7 @@ from pydantic.deprecated.tools import parse_obj_as, schema_json_of, schema_of
 pytestmark = pytest.mark.filterwarnings('ignore::DeprecationWarning')
 
 
-@pytest.mark.parametrize('obj,type_,parsed', [('1', int, 1), (['1'], List[int], [1])])
+@pytest.mark.parametrize('obj,type_,parsed', [('1', int, 1), (['1'], list[int], [1])])
 def test_parse_obj(obj, type_, parsed):
     assert parse_obj_as(type_, obj) == parsed
 
@@ -33,7 +34,7 @@ def test_parse_obj_preserves_subclasses():
 
     model_b = ModelB(a={1: 'f'}, b=2)
 
-    parsed = parse_obj_as(List[ModelA], [model_b])
+    parsed = parse_obj_as(list[ModelA], [model_b])
     assert parsed == [model_b]
 
 
@@ -44,7 +45,7 @@ def test_parse_obj_fails():
         {
             'input': 'a',
             'loc': (),
-            'msg': 'Input should be a valid integer, unable to parse string as an ' 'integer',
+            'msg': 'Input should be a valid integer, unable to parse string as an integer',
             'type': 'int_parsing',
         }
     ]
@@ -72,7 +73,7 @@ def test_parse_as_dataclass():
 
 def test_parse_mapping_as():
     inputs = {'1': '2'}
-    assert parse_obj_as(Dict[int, int], inputs) == {1: 2}
+    assert parse_obj_as(dict[int, int], inputs) == {1: 2}
 
 
 def test_schema():
